import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { QueryHandler } from '@nestjs/cqrs';
import { ConfigurationMessagerPresenter } from '../../adapters/secondaries/configuration-messager.presenter';
import { Messager } from '../../adapters/secondaries/messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { PropagateConfigurationsQuery } from '../../queries/propagate-configurations.query';
import { Configuration } from '../entities/configuration';

@QueryHandler(PropagateConfigurationsQuery)
export class PropagateConfigurationsUseCase {
  constructor(
    private readonly _repository: ConfigurationRepository,
    private readonly _messager: Messager,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  async execute(propagateConfigurationsQuery: PropagateConfigurationsQuery) {
    try {
      const configurations = await this._repository.findAll(1, 999999);
      this._messager.publish(
        'configuration.propagate',
        JSON.stringify(
          configurations.data.map((configuration) =>
            this._mapper.map(
              configuration,
              Configuration,
              ConfigurationMessagerPresenter,
            ),
          ),
        ),
      );
      this._messager.publish(
        'logging.configuration.update.info',
        'propagation',
      );
    } catch (error) {
      this._messager.publish(
        'logging.configuration.update.crit',
        'propagation',
      );
      throw error;
    }
  }
}
