import { NotFoundException } from '@nestjs/common';
import { Test, TestingModule } from '@nestjs/testing';
import { Messager } from '../../adapters/secondaries/messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { Domain } from '../../domain/dtos/domain.enum';
import { FindConfigurationByUuidRequest } from '../../domain/dtos/find-configuration-by-uuid.request';
import { FindConfigurationByUuidUseCase } from '../../domain/usecases/find-configuration-by-uuid.usecase';
import { FindConfigurationByUuidQuery } from '../../queries/find-configuration-by-uuid.query';

const mockConfiguration = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
  domain: Domain.USER,
  key: 'key1',
  value: 'value1',
};

const mockConfigurationRepository = {
  findOneByUuid: jest
    .fn()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementationOnce((query?: FindConfigurationByUuidQuery) => {
      return Promise.resolve(mockConfiguration);
    })
    .mockImplementation(() => {
      return Promise.resolve(undefined);
    }),
};

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};

describe('FindConfigurationByUuidUseCase', () => {
  let findConfigurationByUuidUseCase: FindConfigurationByUuidUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        {
          provide: ConfigurationRepository,
          useValue: mockConfigurationRepository,
        },
        FindConfigurationByUuidUseCase,
        {
          provide: Messager,
          useValue: mockMessager,
        },
      ],
    }).compile();

    findConfigurationByUuidUseCase = module.get<FindConfigurationByUuidUseCase>(
      FindConfigurationByUuidUseCase,
    );
  });

  it('should be defined', () => {
    expect(findConfigurationByUuidUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should return a Configuration', async () => {
      const findConfigurationByUuidRequest: FindConfigurationByUuidRequest =
        new FindConfigurationByUuidRequest();
      findConfigurationByUuidRequest.uuid =
        'bb281075-1b98-4456-89d6-c643d3044a91';
      const Configuration = await findConfigurationByUuidUseCase.execute(
        new FindConfigurationByUuidQuery(findConfigurationByUuidRequest),
      );
      expect(Configuration).toBe(mockConfiguration);
    });
    it('should throw an error if Configuration does not exist', async () => {
      const findConfigurationByUuidRequest: FindConfigurationByUuidRequest =
        new FindConfigurationByUuidRequest();
      findConfigurationByUuidRequest.uuid =
        'bb281075-1b98-4456-89d6-c643d3044a90';
      await expect(
        findConfigurationByUuidUseCase.execute(
          new FindConfigurationByUuidQuery(findConfigurationByUuidRequest),
        ),
      ).rejects.toBeInstanceOf(NotFoundException);
    });
  });
});
